<?php

/**
 * @copyright Copyright (c) Metaways Infosystems GmbH, 2013
 * @license LGPLv3, http://www.arcavias.com/en/license
 */


/**
 * Adds config column to catalog table.
 */
class MW_Setup_Task_CatalogAddConfig extends MW_Setup_Task_Abstract
{
	private $_mysql = 'ALTER TABLE "mshop_catalog" ADD "config" TEXT NOT NULL AFTER "label"';


	/**
	 * Returns the list of task names which this task depends on.
	 *
	 * @return array List of task names
	 */
	public function getPreDependencies()
	{
		return array( 'CatalogTreeToCatalog' );
	}


	/**
	 * Returns the list of task names which depends on this task.
	 *
	 * @return array List of task names
	 */
	public function getPostDependencies()
	{
		return array( 'TablesCreateMShop' );
	}


	/**
	 * Executes the task for MySQL databases.
	 */
	protected function _mysql()
	{
		$this->_process( $this->_mysql );
	}

	/**
	 * Add column to table if it doesn't exist.
	 *
	 * @param string $stmt SQL statement to execute for adding columns
	 */
	protected function _process( $stmt )
	{
		$this->_msg( 'Adding config column to mshop_catalog', 0 );

		if( $this->_schema->tableExists( 'mshop_catalog' ) === true
			&& $this->_schema->columnExists( 'mshop_catalog', 'config' ) === false )
		{
			$this->_execute( $stmt );
			$this->_status( 'added' );
		}
		else
		{
			$this->_status( 'OK' );
		}
	}
}